@@ -1,11 +1,11 @@
 from nrgten.encom import ENCoM
 from nrgten.anm import ANM
 import numpy as np
-import sys
-import glob
-import os
-from scipy.stats import pearsonr
-from multiprocessing import Pool, current_process
+# import sys
+# import glob
+# import os
+# from scipy.stats import pearsonr
+# from multiprocessing import Pool, current_process
 
 
 def get_transit_probs(enm_a, enm_b, gamma=1, alignment=None):
@@ -129,7 +129,8 @@ def cumulative_overlap(reference, target, n_modes, alignment=None, filter=None):
     """Cumulative overlap, as defined in Zimmerman and Jernigan 2014, Eq. 4.
 
     This computes the cumulative overlap between the `n_modes` first normal modes of the reference and the
-    conformational change going from reference to target. See Eq. 4 from `https://doi.org/﻿10.1261/rna.041269.113`
+    conformational change going from reference to target. See Eq. 4 from
+    `doi.org/10.1261/rna.041269.113 <https://doi.org/10.1261/rna.041269.113>`_
 
     Args:
         reference (ENM): The reference ENM object to which the target will be fitted.
@@ -142,6 +143,10 @@ def cumulative_overlap(reference, target, n_modes, alignment=None, filter=None):
     Returns:
         float: the cumulative overlap, using the `n_modes` first normal modes
     """
+    if alignment is None:
+        assert reference.get_n_masses() == target.get_n_masses()
+        n = reference.get_n_masses()
+        alignment = [[x for x in range(n)], [x for x in range(n)]]
     conf_change = fit(reference, target, alignment, filter)
     eigvecs = np.transpose(reference.get_filtered_eigvecs_mat(alignment[0], filter))
     assert len(conf_change) == len(eigvecs[0])
@@ -197,17 +202,39 @@ def rmsd_alignment(reference, target, alignment, filter):
     return rmsd
 
 
-def _pca_ensemble(enm, macromols_list=None, variance_to_explain=0.99, filter=None):
-    """ Principal component analysis on an ensemble of structures. Returns the necessary PCAs to explain the target
-        proportion of variance and their respective proportions of variance explained as:
-        (explained variances, PCAs) (1 PCA per row).
+def pca_ensemble(enm, macromols_list=None, variance_to_explain=0.99, filter=None):
+    """Principal component analysis on an ensemble of structures.
+
+    Args:
+        enm (ENM): An ENM object from which the starting structure will be taken. If the macromols_list optional
+                   argument is None, this object must also contain at least one other state of the macromolecule (the
+                   PDB file must be in NMR format with MODEL and ENDMDL records).
+        macromols_list (list, optional): A list of Macromol objects which will be used to compute the PCs.
+        variance_to_explain (float, optional): The target proportion of variance explained by the PCs.
+        filter (set, optional): set containing the names (str) of the center atoms for the masses to keep.
+
+    Warning:
+        The structures other than the starting structures are restored to their initial states at the end but are
+        temporarily disturbed during the computation. This could lead to nasty effects if parallel computing is
+        attempted without care.
+
+    Returns:
+        tuple: tuple containing:
+            proportion_variance(numpy.ndarray): the proportion of variance explained by each PC
+
+            components(numpy.ndarray): the PCs in matrix notation, 1 PC per row
     """
     if macromols_list is None:
         macromols_list = enm.mols[1:]
     refmol = enm.mol
+    refmol_coords = refmol.get_3n_vector()
+    ref_mean = get_mean_xyz(refmol_coords)
+    refmol.translate_xyz(ref_mean)
+
     if filter is None:
         coords = np.zeros((3*len(refmol.masses), len(macromols_list)+1))
     else:
+        raise ValueError("filtering not yet supported for PCA ensembles")
         coords = np.zeros((3*refmol.get_filtered_n(filter), len(macromols_list) + 1))
     for i, mol in enumerate([refmol] + macromols_list):
         transvec, r = None, None
@@ -218,6 +245,7 @@ def _pca_ensemble(enm, macromols_list=None, variance_to_explain=0.99, filter=Non
         if filter is None:
             coords[:,i] = mol.get_3n_vector()
         else:
+            raise ValueError("filtering not yet supported for PCA ensembles")
             coords[:, i] = mol.get_filtered_3n_vector(filter)
         if i > 0:
             mol.translate_xyz(-transvec)
@@ -232,6 +260,8 @@ def _pca_ensemble(enm, macromols_list=None, variance_to_explain=0.99, filter=Non
     eigvecs = np.transpose(eigvecsmat)
     s = np.sum(eigvals)
     proportion_variance = eigvals/s
+    if variance_to_explain == "all":
+        return np.flip(proportion_variance, axis=0), np.flip(eigvecs, axis=0)
     total_var = 0
     n_significant = None
     for i in range(len(proportion_variance)-1, -1, -1):
@@ -241,6 +271,113 @@ def _pca_ensemble(enm, macromols_list=None, variance_to_explain=0.99, filter=Non
             break
     return np.flip(proportion_variance, axis=0)[0:n_significant], np.flip(eigvecs, axis=0)[0:n_significant]
 
+
+def get_mean_xyz(vector_3n):
+    assert len(vector_3n) % 3 == 0
+    mean_xyz = np.zeros(3)
+    for offset in [0, 1, 2]:
+        for i in range(0, len(vector_3n), 3):
+            mean_xyz[offset] += vector_3n[i+offset]
+    mean_xyz /= (len(vector_3n)/3)
+    return mean_xyz
+
+
+def get_pcs_no_rot_tran(enm_model, proportion_nrt_variance=0.99):
+    assert len(enm_model.mols) > 1
+    if enm_model.eigvals is None:
+        enm_model.solve()
+    variances, pcs = pca_ensemble(enm_model, variance_to_explain="all")
+    rot_tran_vecs = enm_model.eigvecs[:6]
+    nrt_vars = np.zeros(len(variances))
+    nrt_vars_sum = np.zeros(len(variances))
+    vals = np.zeros(len(variances))
+    for i, pc in enumerate(pcs):
+        val = rmsip([pc], rot_tran_vecs)
+        # val = rmsip(rot_tran_vecs, [pc])
+        vals[i] = val
+        nrt_vars[i] = variances[i] * (1 - val)
+    nrt_vars /= np.sum(nrt_vars)
+    nrt_vars, pcs = reorder_pcs(pcs, nrt_vars)
+
+    n_vecs = None
+    for i in range(len(nrt_vars)):
+        nrt_vars_sum[i] = np.sum(nrt_vars[:i+1])
+        if nrt_vars_sum[i] >= proportion_nrt_variance:
+            n_vecs = i+1
+            break
+    if n_vecs is None:
+        raise ValueError("Not able to get to target variance proportion")
+    max_vecs = len(pcs)
+    n_target_vecs = n_vecs+6
+    if n_target_vecs > max_vecs:
+        n_target_vecs = max_vecs
+    return nrt_vars[:n_vecs], _gram_schmidt(np.append(rot_tran_vecs, pcs, axis=0), n_target_vecs)[6:]
+
+
+def reorder_pcs(pcs, nrt_vars):
+    all_array = np.zeros((len(pcs), len(pcs[0])+1))
+    all_array[:, 0] = nrt_vars
+    all_array[:, 1:] = pcs
+    sorted_array = all_array[np.argsort(-all_array[:, 0])]
+    return sorted_array[:, 0], sorted_array[:, 1:]
+
+
+def _gram_schmidt(vectors, target_n):
+    # TODO : apply this to RMSIP to get rid of rot/trans contibution
+    """ From https://gist.github.com/iizukak/1287876/edad3c337844fac34f7e56ec09f9cb27d4907cc7#gistcomment-1871542.
+        Gram-schmidt orthonormalization of row vectors.
+    """
+    basis = []
+    for v in vectors:
+        x = np.zeros((len(v)))
+        for b in basis:
+            x += np.dot(v, b)*b
+        w = v - x
+        # w = v - np.sum(np.dot(v,b)*b  for b in basis) # Calling np.sum(generator) is deprecated
+        if (w > 1e-10).any():
+            basis.append(w/np.linalg.norm(w))
+        if len(basis) == target_n:
+            break
+    if len(basis) < target_n:
+        raise ValueError("Unable to produce {0} orthonormalized vectors".format(target_n))
+    return np.array(basis)
+
+
+def rmsip(ref_vectors, vectors_list):
+    """Root mean square inner product, as defined in Leo-Macias et al. 2005.
+
+    Link to the paper: `doi.org/10.1529/biophysj.104.052449 <https://doi.org/10.1529/biophysj.104.052449>`_
+
+    Args:
+        ref_vectors (np.ndarray): the reference vectors (PCs), in matrix form with one vector per row.
+        vectors_list (np.ndarray): the vectors to test (eigenvectors describing normal modes) in matrix form with one
+                                   vector per row.
+    Returns:
+        float: The root mean square inner product (RMSIP).
+    """
+    s = 0
+    for r in ref_vectors:
+        for v in vectors_list:
+            s += np.square(np.dot(r, v))
+    return np.sqrt(s/len(ref_vectors))
+
+
+def nco(ref_vectors, ref_variances, vectors_list):
+    ref_norms = np.zeros(len(ref_vectors))
+    norms_list = np.zeros(len(vectors_list))
+    for j, vec in enumerate(vectors_list):
+        norms_list[j] = np.linalg.norm(vec)
+    nco_result = []
+    for i, ref_vec in enumerate(ref_vectors):
+        nco_result.append([])
+        ref_norms[i] = np.linalg.norm(ref_vec)
+        overlaps_sq = np.zeros(len(vectors_list))
+        for j, vec in enumerate(vectors_list):
+            overlaps_sq[j] = (np.dot(vec, ref_vec)/(norms_list[j]*ref_norms[i]))**2
+            nco_result[i].append(ref_variances[i]*np.sqrt(np.sum(overlaps_sq[:j+1])))
+    return nco_result
+
+
 def _fit_to_reference_macromol_DEPRECATED(ref, target):
     rmsd, r, transvec = rmsd_kabsch(ref.masscoords, target.masscoords)
     target.rotate(r)
@@ -256,15 +393,6 @@ def _cumulative_overlap_helper(ref_vector, vectors_list):
     return np.sqrt(s)
 
 
-def _rmsip(ref_vectors, vectors_list):
-    """ Root mean square inner product, as defined in Leo-Macis et al. 2005.
-    """
-    s = 0
-    for r in ref_vectors:
-        for v in vectors_list:
-            s += np.square(np.dot(r, v))
-    return np.sqrt(s/len(ref_vectors))
-
 
 def _write_model(enm, count, fh):
     """ Writes one model to the given filehandle, with model number count.
@@ -279,19 +407,23 @@ def _motion(enm, mode_index, stepsize, maxrmsd, filename):
         Important note: mode_index starts at 1, which is the first nontrivial motion (7th normal mode).
     """
     mode = np.copy(enm.eigvecs[mode_index+5])
-    modermsd = _rmsd_of_3n_vector(mode)
-    mode *= stepsize/modermsd
-    nsteps = int(maxrmsd/stepsize)
+    _motion_3n_vector(enm, mode, stepsize, maxrmsd, filename)
+
+
+def _motion_3n_vector(enm, vector_3n, stepsize, maxrmsd, filename):
+    modermsd = _rmsd_of_3n_vector(vector_3n)
+    vector_3n *= stepsize / modermsd
+    nsteps = int(maxrmsd / stepsize)
     count = 1
     offset = 0
     with open(filename, "w") as fh:
         _write_model(enm, count, fh)
         for i, sign in enumerate([1, -1, 1, -1]):
-            mode *= sign
+            vector_3n *= sign
             if i == 3:
                 offset = 1
             for i in range(nsteps - offset):
-                enm._translate_3n_vector(mode)
+                enm._translate_3n_vector(vector_3n)
                 _write_model(enm, count, fh)
                 count += 1
 
@@ -388,10 +520,10 @@ if __name__ == "__main__":
     # test_boltzmann_squares()
 
 
-    open = ENCoM("../../tests/open_clean.pdb")
-    closed = ENCoM("../../tests/closed_clean.pdb")
-    print(get_amplitudes_for_fit(open, closed, 10))
-    # print(overlap(open, closed, 10))
-    # print(overlap(closed, open, 10))
-    # print(open.compute_bfactors()[:15])
+    open_state = ENCoM("../../tests/open_clean.pdb")
+    closed_state = ENCoM("../../tests/closed_clean.pdb")
+    print(get_amplitudes_for_fit(open_state, closed_state, 10))
+    # print(overlap(open_state, closed_state, 10))
+    # print(overlap(closed_state, open_state, 10))
+    # print(open_state.compute_bfactors()[:15])
 
